
package View;

import javax.swing.*;
import Model.*;
import java.awt.*;

/**
 * Kursverlauf zum Veranschaulichen von Brsenkursen
 * Die zu veranschaulichende Aktie hat die Einzelkurse
 * in einem Feld des Datentyps int mit dem Namen verlauf[] gespeichert.
 */

public class Kursverlauf extends JPanel {

    //Aktuelle Aktie
    private Aktie ael;
    
    //maximale Amplitude in Kurspunkten
    private int maxAmplitude;

    //Nullpunkt des Koordinatensystems (Bereich)
    private int ber;
    
    //Stauchung, Streckung in x Richtung (Default: 72)
    private int xAb= 72;
    
    //Stauchung, Strechung in y Richtung (Default: 100)
    private int yAb= 100;
    
    //Abstand zwischen maxAmplitude und Nullpunkt
    private int nAb= yAb/4;


    //Konstruktor der Klasse
    /**
     * 
     * @param aktie Aktie
     */
    public Kursverlauf(Aktie aktie){

        setBackground(Color.white);
        setForeground(Color.black);
        ael= aktie;
        ber= aktie.bereichskurs;

    }
    

    //Paint-Methode
    public void paintComponent(Graphics g){
        
        super.paintComponent(g);
        g.translate(50, 30);

        variablenBerechnen();
        
        koordinatensystemZeichnen(g);
        kursLinieZeichnen(g);

    }

    
    //
    /**
     * Zeichnet das KoordinatenSystem
     * @param g Graphics
     */
    public void koordinatensystemZeichnen(Graphics g){
        
        g.setColor(Color.black);
        
        //Koordinatenachsen zeichnen
        g.drawLine(0, 0, 0, 2*yAb+2*nAb);
        g.drawLine(0, yAb+nAb, (ael.verlauf.length-1)*xAb+nAb, yAb+nAb);

        //y-Achsen Skala
        g.drawString("_", -3, (yAb/10)*0+nAb-2);
        g.drawString("_", -3, (yAb/10)*1+nAb-2);
        g.drawString("_", -3, (yAb/10)*2+nAb-2);
        g.drawString("_", -3, (yAb/10)*3+nAb-2);
        g.drawString("_", -3, (yAb/10)*4+nAb-2);
        g.drawString("_", -3, (yAb/10)*5+nAb-2);
        g.drawString("_", -3, (yAb/10)*6+nAb-2);
        g.drawString("_", -3, (yAb/10)*7+nAb-2);
        g.drawString("_", -3, (yAb/10)*8+nAb-2);
        g.drawString("_", -3, (yAb/10)*9+nAb-2);

        g.drawString("_", -3, (yAb/10)*11+nAb-2);
        g.drawString("_", -3, (yAb/10)*12+nAb-2);
        g.drawString("_", -3, (yAb/10)*13+nAb-2);
        g.drawString("_", -3, (yAb/10)*14+nAb-2);
        g.drawString("_", -3, (yAb/10)*15+nAb-2);
        g.drawString("_", -3, (yAb/10)*16+nAb-2);
        g.drawString("_", -3, (yAb/10)*17+nAb-2);
        g.drawString("_", -3, (yAb/10)*18+nAb-2);
        g.drawString("_", -3, (yAb/10)*19+nAb-2);
        g.drawString("_", -3, (yAb/10)*20+nAb-2);

        //y-Achsen Beschriftung
        g.drawString(""+(int)(ber+maxAmplitude), -40, nAb+4);
        g.drawString(""+ber, -40, yAb+nAb+4);
        g.drawString(""+(int)(ber-maxAmplitude), -40, 2*yAb+nAb+4);
        
        //x-Achsen Skala
        for(int i=1; i <= (ael.verlauf.length-1); i++){
            g.drawString("|", i*xAb, yAb+nAb+5);
        }         

    }
    
    
    //
    /**
     * Zeichnet die Kurslinie in das Koordinatensystem
     * @param g Graphics
     */
    public void kursLinieZeichnen(Graphics g){
        
        g.setColor(Color.BLUE);
        g.drawString("Kurseverlauf", xAb*4-xAb, 0);
        
        //y-Koordiate in Bildpunkten wird fr jeden Einzelkurs berechnet und in diesem Array abgespeichert
        float[] m= new float[ael.verlauf.length+1];

        //Berechnen des ersten Bildpunktes an der y-Achse
        int bildpunktLetzter= 0;
        if(ael.verlauf[ael.verlauf.length-1] <= ber){
            bildpunktLetzter=  (int)((ber- ael.verlauf[ael.verlauf.length-1])*yAb)/(maxAmplitude)+yAb+nAb;
        }
        else if(ael.verlauf[ael.verlauf.length-1] > ber){
            bildpunktLetzter=  yAb+nAb-(int)((ael.verlauf[ael.verlauf.length-1]- ber)*yAb)/(maxAmplitude);
        }
        m[ael.verlauf.length-1]= bildpunktLetzter;

        //Berechnen des Bildpunkts des nchsten Einzelkurses und Linie von BP-vorher bis BP-aktuell zeichnen
        for (int i= ael.verlauf.length-2; i>=0; i--){
            if(ael.verlauf[i] <= ber){
                m[i]= ((ber- ael.verlauf[i])*yAb)/(maxAmplitude)+yAb+nAb;
            }
            else if(ael.verlauf[i] > ber){
                m[i]= yAb+nAb-((ael.verlauf[i]- ber)*yAb)/(maxAmplitude);
            }
            g.drawLine((ael.verlauf.length-2-i)*xAb, (int)m[i+1], (ael.verlauf.length-1-i)*xAb, (int)m[i]);
        }

    }
    
    //
    /**
     * Berechnet die maximale Amplitude aller Einzelkurse
     */
    public void variablenBerechnen(){

        //Berechnet den kleinsten Einzelkurs
        int kleinsterKurs= ber* 9999;
        for(int i=0; i<ael.verlauf.length; i++){
            if(ael.verlauf[i]< kleinsterKurs){
                kleinsterKurs= ael.verlauf[i];
            }
            if(ael.kursGeben()< kleinsterKurs){
                kleinsterKurs= ael.kursGeben();
            }
        }

        //Berechnet den grten Einzelkurs
        int grterKurs= 0;
        for(int i=0; i< ael.verlauf.length; i++){
            if(ael.verlauf[i] > grterKurs){
                grterKurs= ael.verlauf[i];
            }
            if(ael.kursGeben()> grterKurs){
                grterKurs= ael.kursGeben();
            }
        }
        
        //Vergleicht grten und kleinsten Einzelkurs
        int m= 0;
        if(grterKurs- ber > ber- kleinsterKurs){
            m= grterKurs- ber;
        }
        else{
            m= ber- kleinsterKurs;
        }
        
        maxAmplitude= m;
        
    }

}
